// arduino.js

(function(ext) {
    var device = null;
    var _rxBuf = [];
    var speeds = {"LOW":2000,"MEDIUM":1000,"HIGH":500};
    var moves = {"Forward":1,"Backward":2,"Left":3,"Right":4};
    var sides = {"LEFT":0,"RIGHT":1};
    var songs = {"CMHDT":1,"HAPPY_BIRTHDAY":2,"JINGLE_BELLS":3};
    var addressing = {"PAIR":0,"ALL":255};
    // Sensor states:
   
  
  ext.resetAll = function(){};
  
  ext.runArduino = function(){
    responseValue();
  };
  ext.sendTo = function(addr){
  runPackage(80,short2array(typeof addr=="number"?addr:addressing[addr]));
  };
  ext.move = function(move,speed) {
      runPackage(60,short2array(typeof move=="number"?move:moves[move]),short2array(typeof speed=="number"?speed:speeds[speed]));
  };

  ext.updown = function(speed) {
    runPackage(61,short2array(typeof speed=="number"?speed:speeds[speed]));
  };

  ext.pushup = function(speed) {
    runPackage(62,short2array(typeof speed=="number"?speed:speeds[speed]));
  };

  ext.dance = function(speed) {
    runPackage(63,short2array(typeof speed=="number"?speed:speeds[speed]));
  };

  ext.hello = function(speed) {
    runPackage(64,short2array(typeof speed=="number"?speed:speeds[speed]));
  };

  ext.home = function() {
    runPackage(65);
  };

  ext.zero = function() {
    runPackage(66);
  };

  ext.sing = function(song) {
    runPackage(67,short2array(typeof song=="number"?song:songs[song]));
  };

  ///////////////////////////////////////////
   ext.getDistance = function(nextID) {
    var deviceId = 71;
    getPackage(nextID,deviceId);
    };
  ext.getLight = function(nextID,side) {
    var deviceId = 72;
    getPackage(nextID,deviceId,typeof side=="number"?side:sides[side]);
    }; 
  ext.actionDone = function(nextID) {
    var deviceId = 92;
    getPackage(nextID,deviceId);
    };   
  ////////////////////////////////////////////
  
   function sendPackage(argList, type){
    var bytes = [0xff, 0x55, 0, 0, type];
    for(var i=0;i<argList.length;++i){
      var val = argList[i];
      if(val.constructor == "[class Array]"){
        bytes = bytes.concat(val);
      }else{
        bytes.push(val);
      }
    }
    bytes[2] = bytes.length - 3;
    device.send(bytes);
  }
  
  function runPackage(){
    sendPackage(arguments, 2);
  }
  function getPackage(){
    var nextID = arguments[0];
    Array.prototype.shift.call(arguments);
    sendPackage(arguments, 1);
  }

    var inputArray = [];
  var _isParseStart = false;
  var _isParseStartIndex = 0;
  
    function processData(bytes) {
    var len = bytes.length;
    if(_rxBuf.length>30){
      _rxBuf = [];
    }
    for(var index=0;index<bytes.length;index++){
      var c = bytes[index];
      _rxBuf.push(c);
      if(_rxBuf.length>=2){
        if(_rxBuf[_rxBuf.length-1]==0x55 && _rxBuf[_rxBuf.length-2]==0xff){
          _isParseStart = true;
          _isParseStartIndex = _rxBuf.length-2;
        }
        if(_rxBuf[_rxBuf.length-1]==0xa && _rxBuf[_rxBuf.length-2]==0xd&&_isParseStart){
          _isParseStart = false;
          
          var position = _isParseStartIndex+2;
          var extId = _rxBuf[position];
          position++;
          var type = _rxBuf[position];
          position++;
          //1 byte 2 float 3 short 4 len+string 5 double
          var value;
          switch(type){
            case 1:{
              value = _rxBuf[position];
              position++;
            }
              break;
            case 2:{
              value = readFloat(_rxBuf,position);
              position+=4;
              if(value<-255||value>1023){
                value = 0;
              }
            }
              break;
            case 3:{
              value = readInt(_rxBuf,position,2);
              position+=2;
            }
              break;
            case 4:{
              var l = _rxBuf[position];
              position++;
              value = readString(_rxBuf,position,l);
            }
              break;
            case 5:{
              value = readDouble(_rxBuf,position);
              position+=4;
            }
              break;
            case 6:
              value = readInt(_rxBuf,position,4);
              position+=4;
              break;
          }
          if(type<=6){
            responseValue(extId,value);
          }else{
            responseValue();
          }
          _rxBuf = [];
        }
      } 
    }
    }
  function readFloat(arr,position){
    var f= [arr[position],arr[position+1],arr[position+2],arr[position+3]];
    return parseFloat(f);
  }
  function readInt(arr,position,count){
    var result = 0;
    for(var i=0; i<count; ++i){
      result |= arr[position+i] << (i << 3);
    }
    return result;
  }
  function readDouble(arr,position){
    return readFloat(arr,position);
  }
  function readString(arr,position,len){
    var value = "";
    for(var ii=0;ii<len;ii++){
      value += String.fromCharCode(_rxBuf[ii+position]);
    }
    return value;
  }
    function appendBuffer( buffer1, buffer2 ) {
        return buffer1.concat( buffer2 );
    }

    // Extension API interactions
    var potentialDevices = [];
    ext._deviceConnected = function(dev) {
        potentialDevices.push(dev);

        if (!device) {
            tryNextDevice();
        }
    }

    function tryNextDevice() {
        // If potentialDevices is empty, device will be undefined.
        // That will get us back here next time a device is connected.
        device = potentialDevices.shift();
        if (device) {
            device.open({ stopBits: 0, bitRate: 115200, ctsFlowControl: 0 }, deviceOpened);
        }
    }

    var watchdog = null;
    function deviceOpened(dev) {
        if (!dev) {
            // Opening the port failed.
            tryNextDevice();
            return;
        }
        device.set_receive_handler('Robot Guti',processData);
    };

    ext._deviceRemoved = function(dev) {
        if(device != dev) return;
        device = null;
    };

    ext._shutdown = function() {
        if(device) device.close();
        device = null;
    };

    ext._getStatus = function() {
        if(!device) return {status: 1, msg: 'Arduino disconnected'};
        if(watchdog) return {status: 1, msg: 'Probing for Arduino'};
        return {status: 2, msg: 'Arduino connected'};
    }

    var descriptor = {};
  ScratchExtensions.register('Robot Guti', descriptor, ext, {type: 'serial'});
})({});
